@@ -9,7 +9,7 @@ use DynaLoader();
 use Carp ();
 @ISA = qw(DynaLoader);
 
-$VERSION = '4.004';
+$VERSION = '4.005';
 
 bootstrap DBD::mysql $VERSION;
 
@@ -254,112 +254,133 @@ sub _SelectDB ($$) {
 }
 
 sub table_info ($) {
-    my ($dbh, $catalog, $schema, $table, $type, $attr) = @_;
-    my @names = qw(TABLE_CAT TABLE_SCHEM TABLE_NAME TABLE_TYPE REMARKS);
-    my @rows;
+  my ($dbh, $catalog, $schema, $table, $type, $attr) = @_;
+  $dbh->{mysql_server_prepare}||= 0;
+  my $mysql_server_prepare_save= $dbh->{mysql_server_prepare};
+  $dbh->{mysql_server_prepare}= 0;
+  my @names = qw(TABLE_CAT TABLE_SCHEM TABLE_NAME TABLE_TYPE REMARKS);
+  my @rows;
 
-    my $sponge = DBI->connect("DBI:Sponge:", '','')
-	or return $dbh->DBI::set_err($DBI::err, "DBI::Sponge: $DBI::errstr");
+  my $sponge = DBI->connect("DBI:Sponge:", '','')
+    or return $dbh->DBI::set_err($DBI::err, "DBI::Sponge: $DBI::errstr");
 
-    # Return the list of catalogs
-    if (defined $catalog && $catalog eq "%" &&
-        (!defined($schema) || $schema eq "") &&
-        (!defined($table) || $table eq ""))
+# Return the list of catalogs
+  if (defined $catalog && $catalog eq "%" &&
+      (!defined($schema) || $schema eq "") &&
+      (!defined($table) || $table eq ""))
+  {
+    @rows = (); # Empty, because MySQL doesn't support catalogs (yet)
+  }
+  # Return the list of schemas
+  elsif (defined $schema && $schema eq "%" &&
+      (!defined($catalog) || $catalog eq "") &&
+      (!defined($table) || $table eq ""))
+  {
+    my $sth = $dbh->prepare("SHOW DATABASES")
+      or ($dbh->{mysql_server_prepare}= $mysql_server_prepare_save && 
+          return undef);
+
+    $sth->execute()
+      or ($dbh->{mysql_server_prepare}= $mysql_server_prepare_save && 
+        return DBI::set_err($dbh, $sth->err(), $sth->errstr()));
+
+    while (my $ref = $sth->fetchrow_arrayref())
     {
-	@rows = (); # Empty, because MySQL doesn't support catalogs (yet)
+      push(@rows, [ undef, $ref->[0], undef, undef, undef ]);
     }
+  }
+  # Return the list of table types
+  elsif (defined $type && $type eq "%" &&
+      (!defined($catalog) || $catalog eq "") &&
+      (!defined($schema) || $schema eq "") &&
+      (!defined($table) || $table eq ""))
+  {
+    @rows = (
+        [ undef, undef, undef, "TABLE", undef ],
+        [ undef, undef, undef, "VIEW",  undef ],
+        );
+  }
+  # Special case: a catalog other than undef, "", or "%"
+  elsif (defined $catalog && $catalog ne "" && $catalog ne "%")
+  {
+    @rows = (); # Nothing, because MySQL doesn't support catalogs yet.
+  }
+  # Uh oh, we actually have a meaty table_info call. Work is required!
+  else
+  {
+    my @schemas;
+    # If no table was specified, we want them all
+    $table ||= "%";
 
-    # Return the list of schemas
-    elsif (defined $schema && $schema eq "%" &&
-           (!defined($catalog) || $catalog eq "") &&
-           (!defined($table) || $table eq ""))
+    # If something was given for the schema, we need to expand it to
+    # a list of schemas, since it may be a wildcard.
+    if (defined $schema && $schema ne "")
     {
-	my $sth = $dbh->prepare("SHOW DATABASES")
-	    or return undef;
+      my $sth = $dbh->prepare("SHOW DATABASES LIKE " .
+          $dbh->quote($schema))
+        or ($dbh->{mysql_server_prepare}= $mysql_server_prepare_save && 
+        return undef);
+      $sth->execute()
+        or ($dbh->{mysql_server_prepare}= $mysql_server_prepare_save && 
+        return DBI::set_err($dbh, $sth->err(), $sth->errstr()));
 
-	$sth->execute()
-	    or return DBI::set_err($dbh, $sth->err(), $sth->errstr());
-
-	while (my $ref = $sth->fetchrow_arrayref()) {
-	    push(@rows, [ undef, $ref->[0], undef, undef, undef ]);
-	}
+      while (my $ref = $sth->fetchrow_arrayref())
+      {
+        push @schemas, $ref->[0];
+      }
     }
-
-    # Return the list of table types
-    elsif (defined $type && $type eq "%" &&
-           (!defined($catalog) || $catalog eq "") &&
-           (!defined($schema) || $schema eq "") &&
-           (!defined($table) || $table eq ""))
+    # Otherwise we want the current database
+    else
     {
-	@rows = (
-		 [ undef, undef, undef, "TABLE", undef ],
-		 [ undef, undef, undef, "VIEW",  undef ],
-		);
+      push @schemas, $dbh->selectrow_array("SELECT DATABASE()");
     }
 
-    # Special case: a catalog other than undef, "", or "%"
-    elsif (defined $catalog && $catalog ne "" && $catalog ne "%") {
-	@rows = (); # Nothing, because MySQL doesn't support catalogs yet.
+    # Figure out which table types are desired
+    my ($want_tables, $want_views);
+    if (defined $type && $type ne "")
+    {
+      $want_tables = ($type =~ m/table/i);
+      $want_views  = ($type =~ m/view/i);
+    }
+    else
+    {
+      $want_tables = $want_views = 1;
     }
 
-    # Uh oh, we actually have a meaty table_info call. Work is required!
-    else {
-	my @schemas;
+    for my $database (@schemas)
+    {
+      my $sth = $dbh->prepare("SHOW /*!50002 FULL*/ TABLES FROM " .
+          $dbh->quote_identifier($database) .
+          " LIKE " .  $dbh->quote($table))
+          or ($dbh->{mysql_server_prepare}= $mysql_server_prepare_save && 
+          return undef);
 
-	# If no table was specified, we want them all
-	$table ||= "%";
+      $sth->execute() or
+          ($dbh->{mysql_server_prepare}= $mysql_server_prepare_save &&
+          return DBI::set_err($dbh, $sth->err(), $sth->errstr()));
 
-	# If something was given for the schema, we need to expand it to
-	# a list of schemas, since it may be a wildcard.
-	if (defined $schema && $schema ne "") {
-	    my $sth = $dbh->prepare("SHOW DATABASES LIKE " .
-				    $dbh->quote($schema))
-		or return undef;
-	    $sth->execute()
-		or return DBI::set_err($dbh, $sth->err(), $sth->errstr());
-
-	    while (my $ref = $sth->fetchrow_arrayref()) {
-		push @schemas, $ref->[0];
-	    }
-	# Otherwise we want the current database
-	} else {
-	    push @schemas, $dbh->selectrow_array("SELECT DATABASE()");
-	}
-
-	# Figure out which table types are desired
-	my ($want_tables, $want_views);
-	if (defined $type && $type ne "") {
-	    $want_tables = ($type =~ m/table/i);
-	    $want_views  = ($type =~ m/view/i);
-	} else {
-	    $want_tables = $want_views = 1;
-	}
-
-	foreach my $database (@schemas) {
-	    my $sth = $dbh->prepare("SHOW /*!50002 FULL*/ TABLES FROM " .
-				    $dbh->quote_identifier($database) .
-				    " LIKE " .  $dbh->quote($table))
-		or return undef;
-	    $sth->execute() or
-		return DBI::set_err($dbh, $sth->err(), $sth->errstr());
-
-	    while (my $ref = $sth->fetchrow_arrayref()) {
-		my $type = (defined $ref->[1] &&
-		            $ref->[1] =~ /view/i) ? 'VIEW' : 'TABLE';
-		next if $type eq 'TABLE' && not $want_tables;
-		next if $type eq 'VIEW'  && not $want_views;
-		push @rows, [ undef, $database, $ref->[0], $type, undef ];
-	    }
-	}
+      while (my $ref = $sth->fetchrow_arrayref())
+      {
+        my $type = (defined $ref->[1] &&
+            $ref->[1] =~ /view/i) ? 'VIEW' : 'TABLE';
+        next if $type eq 'TABLE' && not $want_tables;
+        next if $type eq 'VIEW'  && not $want_views;
+        push @rows, [ undef, $database, $ref->[0], $type, undef ];
+      }
     }
+  }
 
-    my $sth = $sponge->prepare("table_info", {
-	rows => \@rows,
-	NUM_OF_FIELDS => scalar @names,
-	NAME => \@names,
-    }) or return $dbh->DBI::set_err($sponge->err(), $sponge->errstr());
+  my $sth = $sponge->prepare("table_info",
+  {
+    rows          => \@rows,
+    NUM_OF_FIELDS => scalar @names,
+    NAME          => \@names,
+  }) 
+    or ($dbh->{mysql_server_prepare}= $mysql_server_prepare_save && 
+      return $dbh->DBI::set_err($sponge->err(), $sponge->errstr()));
 
-    return $sth;
+  $dbh->{mysql_server_prepare}= $mysql_server_prepare_save;
+  return $sth;
 }
 
 sub _ListTables {
@@ -372,199 +393,240 @@ sub _ListTables {
 
 
 sub column_info {
-    my ($dbh, $catalog, $schema, $table, $column) = @_;
-    # ODBC allows a NULL to mean all columns, so we'll accept undef
-    $column = '%' unless defined $column;
+  my ($dbh, $catalog, $schema, $table, $column) = @_;
+  $dbh->{mysql_server_prepare}||= 0;
+  my $mysql_server_prepare_save= $dbh->{mysql_server_prepare};
+  $dbh->{mysql_server_prepare}= 0;
 
-    my $ER_NO_SUCH_TABLE= 1146;
+  # ODBC allows a NULL to mean all columns, so we'll accept undef
+  $column = '%' unless defined $column;
 
-    my $table_id = $dbh->quote_identifier($catalog, $schema, $table);
+  my $ER_NO_SUCH_TABLE= 1146;
 
-    my @names = qw(
-	TABLE_CAT TABLE_SCHEM TABLE_NAME COLUMN_NAME
-	DATA_TYPE TYPE_NAME COLUMN_SIZE BUFFER_LENGTH DECIMAL_DIGITS
-	NUM_PREC_RADIX NULLABLE REMARKS COLUMN_DEF
-	SQL_DATA_TYPE SQL_DATETIME_SUB CHAR_OCTET_LENGTH
-	ORDINAL_POSITION IS_NULLABLE CHAR_SET_CAT
-	CHAR_SET_SCHEM CHAR_SET_NAME COLLATION_CAT COLLATION_SCHEM COLLATION_NAME
-	UDT_CAT UDT_SCHEM UDT_NAME DOMAIN_CAT DOMAIN_SCHEM DOMAIN_NAME
-	SCOPE_CAT SCOPE_SCHEM SCOPE_NAME MAX_CARDINALITY
-	DTD_IDENTIFIER IS_SELF_REF
-	mysql_is_pri_key mysql_type_name mysql_values
-        mysql_is_auto_increment
-    );
-    my %col_info;
+  my $table_id = $dbh->quote_identifier($catalog, $schema, $table);
 
-    local $dbh->{FetchHashKeyName} = 'NAME_lc';
-    # only ignore ER_NO_SUCH_TABLE in internal_execute if issued from here
-    my $desc_sth = $dbh->prepare("DESCRIBE $table_id " . $dbh->quote($column));
-    my $desc = $dbh->selectall_arrayref($desc_sth, { Columns=>{} });
+  my @names = qw(
+      TABLE_CAT TABLE_SCHEM TABLE_NAME COLUMN_NAME
+      DATA_TYPE TYPE_NAME COLUMN_SIZE BUFFER_LENGTH DECIMAL_DIGITS
+      NUM_PREC_RADIX NULLABLE REMARKS COLUMN_DEF
+      SQL_DATA_TYPE SQL_DATETIME_SUB CHAR_OCTET_LENGTH
+      ORDINAL_POSITION IS_NULLABLE CHAR_SET_CAT
+      CHAR_SET_SCHEM CHAR_SET_NAME COLLATION_CAT COLLATION_SCHEM COLLATION_NAME
+      UDT_CAT UDT_SCHEM UDT_NAME DOMAIN_CAT DOMAIN_SCHEM DOMAIN_NAME
+      SCOPE_CAT SCOPE_SCHEM SCOPE_NAME MAX_CARDINALITY
+      DTD_IDENTIFIER IS_SELF_REF
+      mysql_is_pri_key mysql_type_name mysql_values
+      mysql_is_auto_increment
+      );
+  my %col_info;
 
-    #return $desc_sth if $desc_sth->err();
-    if (my $err = $desc_sth->err())
+  local $dbh->{FetchHashKeyName} = 'NAME_lc';
+  # only ignore ER_NO_SUCH_TABLE in internal_execute if issued from here
+  my $desc_sth = $dbh->prepare("DESCRIBE $table_id " . $dbh->quote($column));
+  my $desc = $dbh->selectall_arrayref($desc_sth, { Columns=>{} });
+
+  #return $desc_sth if $desc_sth->err();
+  if (my $err = $desc_sth->err())
+  {
+    # return the error, unless it is due to the table not 
+    # existing per DBI spec
+    if ($err != $ER_NO_SUCH_TABLE)
     {
-        # return the error, unless it is due to the table not 
-        # existing per DBI spec
-        if ($err != $ER_NO_SUCH_TABLE)
-        {
-          return undef;
-        }
-        $dbh->set_err(undef,undef);
-        $desc = [];
+      $dbh->{mysql_server_prepare}= $mysql_server_prepare_save;
+      return undef;
     }
+    $dbh->set_err(undef,undef);
+    $desc = [];
+  }
 
-    my $ordinal_pos = 0;
-    foreach my $row (@$desc) {
-	my $type = $row->{type};
-	$type =~ m/^(\w+)(?:\((.*?)\))?\s*(.*)/;
-	my $basetype = lc($1);
-        my $typemod = $2;
-        my $attr = $3;
+  my $ordinal_pos = 0;
+  for my $row (@$desc)
+  {
+    my $type = $row->{type};
+    $type =~ m/^(\w+)(?:\((.*?)\))?\s*(.*)/;
+    my $basetype  = lc($1);
+    my $typemod   = $2;
+    my $attr      = $3;
 
-	my $info = $col_info{ $row->{field} } = {
-	    TABLE_CAT   => $catalog,
-	    TABLE_SCHEM => $schema,
-	    TABLE_NAME  => $table,
-	    COLUMN_NAME => $row->{field},
-	    NULLABLE    => ($row->{null} eq 'YES') ? 1 : 0,
-	    IS_NULLABLE => ($row->{null} eq 'YES') ? "YES" : "NO",
-	    TYPE_NAME   => uc($basetype),
-	    COLUMN_DEF  => $row->{default},
-	    ORDINAL_POSITION => ++$ordinal_pos,
-	    mysql_is_pri_key => ($row->{key}  eq 'PRI'),
-	    mysql_type_name  => $row->{type},
-            mysql_is_auto_increment => ($row->{extra} =~ /auto_increment/i ? 1 :
-                                        0),
-	};
-	# This code won't deal with a pathalogical case where a value
-	# contains a single quote followed by a comma, and doesn't unescape
-	# any escaped values. But who would use those in an enum or set?
-	my @type_params = ($typemod && index($typemod,"'")>=0)
-			? ("$typemod," =~ /'(.*?)',/g)  # assume all are quoted
+    my $info = $col_info{ $row->{field} }= {
+	    TABLE_CAT               => $catalog,
+	    TABLE_SCHEM             => $schema,
+	    TABLE_NAME              => $table,
+	    COLUMN_NAME             => $row->{field},
+	    NULLABLE                => ($row->{null} eq 'YES') ? 1 : 0,
+	    IS_NULLABLE             => ($row->{null} eq 'YES') ? "YES" : "NO",
+	    TYPE_NAME               => uc($basetype),
+	    COLUMN_DEF              => $row->{default},
+	    ORDINAL_POSITION        => ++$ordinal_pos,
+	    mysql_is_pri_key        => ($row->{key}  eq 'PRI'),
+	    mysql_type_name         => $row->{type},
+      mysql_is_auto_increment => ($row->{extra} =~ /auto_increment/i ? 1 : 0),
+    };
+    #
+	  # This code won't deal with a pathalogical case where a value
+	  # contains a single quote followed by a comma, and doesn't unescape
+	  # any escaped values. But who would use those in an enum or set?
+    #
+	  my @type_params= ($typemod && index($typemod,"'")>=0) ?
+      ("$typemod," =~ /'(.*?)',/g)  # assume all are quoted
 			: split /,/, $typemod||'';      # no quotes, plain list
-	s/''/'/g for @type_params;                # undo doubling of quotes
-	my @type_attr = split / /, $attr||'';
-	#warn "$type: $basetype [@type_params] [@type_attr]\n";
+	  s/''/'/g for @type_params;                # undo doubling of quotes
 
-	$info->{DATA_TYPE} = SQL_VARCHAR();
-	if ($basetype =~ /^(char|varchar|\w*text|\w*blob)/) {
-	    $info->{DATA_TYPE} = SQL_CHAR() if $basetype eq 'char';
-	    if ($type_params[0]) {
-		$info->{COLUMN_SIZE} = $type_params[0];
-	    }
-	    else {
-		$info->{COLUMN_SIZE} = 65535;
-		$info->{COLUMN_SIZE} = 255        if $basetype =~ /^tiny/;
-		$info->{COLUMN_SIZE} = 16777215   if $basetype =~ /^medium/;
-		$info->{COLUMN_SIZE} = 4294967295 if $basetype =~ /^long/;
-	    }
-	}
-	elsif ($basetype =~ /^(binary|varbinary)/) {
-	    $info->{COLUMN_SIZE} = $type_params[0];
+	  my @type_attr= split / /, $attr||'';
+
+  	$info->{DATA_TYPE}= SQL_VARCHAR();
+    if ($basetype =~ /^(char|varchar|\w*text|\w*blob)/)
+    {
+      $info->{DATA_TYPE}= SQL_CHAR() if $basetype eq 'char';
+      if ($type_params[0])
+      {
+        $info->{COLUMN_SIZE} = $type_params[0];
+      }
+      else
+      {
+        $info->{COLUMN_SIZE} = 65535;
+        $info->{COLUMN_SIZE} = 255        if $basetype =~ /^tiny/;
+        $info->{COLUMN_SIZE} = 16777215   if $basetype =~ /^medium/;
+        $info->{COLUMN_SIZE} = 4294967295 if $basetype =~ /^long/;
+      }
+    }
+	  elsif ($basetype =~ /^(binary|varbinary)/)
+    {
+      $info->{COLUMN_SIZE} = $type_params[0];
 	    # SQL_BINARY & SQL_VARBINARY are tempting here but don't match the
 	    # semantics for mysql (not hex). SQL_CHAR &  SQL_VARCHAR are correct here.
 	    $info->{DATA_TYPE} = ($basetype eq 'binary') ? SQL_CHAR() : SQL_VARCHAR();
-	}
-	elsif ($basetype =~ /^(enum|set)/) {
-	    if ($basetype eq 'set') {
-		$info->{COLUMN_SIZE} = length(join ",", @type_params);
+    }
+    elsif ($basetype =~ /^(enum|set)/)
+    {
+	    if ($basetype eq 'set')
+      {
+		    $info->{COLUMN_SIZE} = length(join ",", @type_params);
 	    }
-	    else {
-		my $max_len = 0;
-		length($_) > $max_len and $max_len = length($_) for @type_params;
-		$info->{COLUMN_SIZE} = $max_len;
+	    else
+      {
+        my $max_len = 0;
+        length($_) > $max_len and $max_len = length($_) for @type_params;
+        $info->{COLUMN_SIZE} = $max_len;
 	    }
 	    $info->{"mysql_values"} = \@type_params;
-	}
-	elsif ($basetype =~ /int/) { # big/medium/small/tiny etc + unsigned?
+    }
+    elsif ($basetype =~ /int/)
+    { 
+      # big/medium/small/tiny etc + unsigned?
 	    $info->{DATA_TYPE} = SQL_INTEGER();
 	    $info->{NUM_PREC_RADIX} = 10;
 	    $info->{COLUMN_SIZE} = $type_params[0];
-	}
-	elsif ($basetype =~ /^decimal/) {
-	    $info->{DATA_TYPE} = SQL_DECIMAL();
-	    $info->{NUM_PREC_RADIX} = 10;
-	    $info->{COLUMN_SIZE}    = $type_params[0];
-	    $info->{DECIMAL_DIGITS} = $type_params[1];
-	}
-	elsif ($basetype =~ /^(float|double)/) {
+    }
+    elsif ($basetype =~ /^decimal/)
+    {
+      $info->{DATA_TYPE} = SQL_DECIMAL();
+      $info->{NUM_PREC_RADIX} = 10;
+      $info->{COLUMN_SIZE}    = $type_params[0];
+      $info->{DECIMAL_DIGITS} = $type_params[1];
+    }
+    elsif ($basetype =~ /^(float|double)/)
+    {
 	    $info->{DATA_TYPE} = ($basetype eq 'float') ? SQL_FLOAT() : SQL_DOUBLE();
 	    $info->{NUM_PREC_RADIX} = 2;
 	    $info->{COLUMN_SIZE} = ($basetype eq 'float') ? 32 : 64;
-	}
-	elsif ($basetype =~ /date|time/) { # date/datetime/time/timestamp
-	    if ($basetype eq 'time' or $basetype eq 'date') {
-		#$info->{DATA_TYPE}   = ($basetype eq 'time') ? SQL_TYPE_TIME() : SQL_TYPE_DATE();
-                $info->{DATA_TYPE}   = ($basetype eq 'time') ? SQL_TIME() : SQL_DATE(); 
-		$info->{COLUMN_SIZE} = ($basetype eq 'time') ? 8 : 10;
-	    }
-	    else { # datetime/timestamp
-		#$info->{DATA_TYPE}     = SQL_TYPE_TIMESTAMP();
-		$info->{DATA_TYPE}     = SQL_TIMESTAMP();
-		$info->{SQL_DATA_TYPE} = SQL_DATETIME();
-	        $info->{SQL_DATETIME_SUB} = $info->{DATA_TYPE} - ($info->{SQL_DATA_TYPE} * 10);
-		$info->{COLUMN_SIZE}   = ($basetype eq 'datetime') ? 19 : $type_params[0] || 14;
-	    }
-	    $info->{DECIMAL_DIGITS} = 0; # no fractional seconds
-	}
-	elsif ($basetype eq 'year') {	# no close standard so treat as int
-	    $info->{DATA_TYPE} = SQL_INTEGER();
-	    $info->{NUM_PREC_RADIX} = 10;
-	    $info->{COLUMN_SIZE} = 4;
-	}
-	else {
-	    Carp::carp("column_info: unrecognized column type '$basetype' of $table_id.$row->{field} treated as varchar");
-	}
-	$info->{SQL_DATA_TYPE} ||= $info->{DATA_TYPE};
-	#warn Dumper($info);
     }
+    elsif ($basetype =~ /date|time/)
+    { 
+      # date/datetime/time/timestamp
+	    if ($basetype eq 'time' or $basetype eq 'date')
+      {
+		    #$info->{DATA_TYPE}   = ($basetype eq 'time') ? SQL_TYPE_TIME() : SQL_TYPE_DATE();
+        $info->{DATA_TYPE}   = ($basetype eq 'time') ? SQL_TIME() : SQL_DATE(); 
+        $info->{COLUMN_SIZE} = ($basetype eq 'time') ? 8 : 10;
+      }
+	    else
+      {
+        # datetime/timestamp
+        #$info->{DATA_TYPE}     = SQL_TYPE_TIMESTAMP();
+		    $info->{DATA_TYPE}        = SQL_TIMESTAMP();
+		    $info->{SQL_DATA_TYPE}    = SQL_DATETIME();
+        $info->{SQL_DATETIME_SUB} = $info->{DATA_TYPE} - ($info->{SQL_DATA_TYPE} * 10);
+        $info->{COLUMN_SIZE}      = ($basetype eq 'datetime') ? 19 : $type_params[0] || 14;
+	    }
+	    $info->{DECIMAL_DIGITS}= 0; # no fractional seconds
+    }
+    elsif ($basetype eq 'year')
+    {	
+      # no close standard so treat as int
+	    $info->{DATA_TYPE}      = SQL_INTEGER();
+	    $info->{NUM_PREC_RADIX} = 10;
+	    $info->{COLUMN_SIZE}    = 4;
+	  }
+	  else
+    {
+	    Carp::carp("column_info: unrecognized column type '$basetype' of $table_id.$row->{field} treated as varchar");
+    }
+    $info->{SQL_DATA_TYPE} ||= $info->{DATA_TYPE};
+    #warn Dumper($info);
+  }
 
-    my $sponge = DBI->connect("DBI:Sponge:", '','')
-	or return $dbh->DBI::set_err($DBI::err, "DBI::Sponge: $DBI::errstr");
-    my $sth = $sponge->prepare("column_info $table", {
-	rows => [ map { [ @{$_}{@names} ] } values %col_info ],
-	NUM_OF_FIELDS => scalar @names,
-	NAME => \@names,
-    }) or return $dbh->DBI::set_err($sponge->err(), $sponge->errstr());
+  my $sponge = DBI->connect("DBI:Sponge:", '','')
+    or (  $dbh->{mysql_server_prepare}= $mysql_server_prepare_save &&
+          return $dbh->DBI::set_err($DBI::err, "DBI::Sponge: $DBI::errstr"));
 
-    return $sth;
+  my $sth = $sponge->prepare("column_info $table", {
+      rows          => [ map { [ @{$_}{@names} ] } values %col_info ],
+      NUM_OF_FIELDS => scalar @names,
+      NAME          => \@names,
+      }) or
+  return ($dbh->{mysql_server_prepare}= $mysql_server_prepare_save &&
+          $dbh->DBI::set_err($sponge->err(), $sponge->errstr()));
+
+  $dbh->{mysql_server_prepare}= $mysql_server_prepare_save;
+  return $sth;
 }
 
 
 sub primary_key_info {
-    my ($dbh, $catalog, $schema, $table) = @_;
+  my ($dbh, $catalog, $schema, $table) = @_;
+  $dbh->{mysql_server_prepare}||= 0;
+  my $mysql_server_prepare_save= $dbh->{mysql_server_prepare};
 
-    my $table_id = $dbh->quote_identifier($catalog, $schema, $table);
+  my $table_id = $dbh->quote_identifier($catalog, $schema, $table);
 
-    my @names = qw(
-	TABLE_CAT TABLE_SCHEM TABLE_NAME COLUMN_NAME KEY_SEQ PK_NAME
-    );
-    my %col_info;
+  my @names = qw(
+      TABLE_CAT TABLE_SCHEM TABLE_NAME COLUMN_NAME KEY_SEQ PK_NAME
+      );
+  my %col_info;
 
-    local $dbh->{FetchHashKeyName} = 'NAME_lc';
-    my $desc_sth = $dbh->prepare("SHOW KEYS FROM $table_id");
-    my $desc = $dbh->selectall_arrayref($desc_sth, { Columns=>{} });
-    my $ordinal_pos = 0;
-    foreach my $row (grep { $_->{key_name} eq 'PRIMARY'} @$desc) {
-	$col_info{ $row->{column_name} } = {
-	    TABLE_CAT   => $catalog,
-	    TABLE_SCHEM => $schema,
-	    TABLE_NAME  => $table,
-	    COLUMN_NAME => $row->{column_name},
-            KEY_SEQ     => $row->{seq_in_index},
-            PK_NAME     => $row->{key_name},
-	};
-    }
+  local $dbh->{FetchHashKeyName} = 'NAME_lc';
+  my $desc_sth = $dbh->prepare("SHOW KEYS FROM $table_id");
+  my $desc= $dbh->selectall_arrayref($desc_sth, { Columns=>{} });
+  my $ordinal_pos = 0;
+  for my $row (grep { $_->{key_name} eq 'PRIMARY'} @$desc)
+  {
+    $col_info{ $row->{column_name} }= {
+      TABLE_CAT   => $catalog,
+      TABLE_SCHEM => $schema,
+      TABLE_NAME  => $table,
+      COLUMN_NAME => $row->{column_name},
+      KEY_SEQ     => $row->{seq_in_index},
+      PK_NAME     => $row->{key_name},
+    };
+  }
 
-    my $sponge = DBI->connect("DBI:Sponge:", '','')
-	or return $dbh->DBI::set_err($DBI::err, "DBI::Sponge: $DBI::errstr");
-    my $sth = $sponge->prepare("primary_key_info $table", {
-	rows => [ map { [ @{$_}{@names} ] } values %col_info ],
-	NUM_OF_FIELDS => scalar @names,
-	NAME => \@names,
-    }) or return $dbh->DBI::set_err($sponge->err(), $sponge->errstr());
+  my $sponge = DBI->connect("DBI:Sponge:", '','')
+    or 
+     ($dbh->{mysql_server_prepare}= $mysql_server_prepare_save &&
+      return $dbh->DBI::set_err($DBI::err, "DBI::Sponge: $DBI::errstr"));
 
-    return $sth;
+  my $sth= $sponge->prepare("primary_key_info $table", {
+      rows          => [ map { [ @{$_}{@names} ] } values %col_info ],
+      NUM_OF_FIELDS => scalar @names,
+      NAME          => \@names,
+      }) or 
+       ($dbh->{mysql_server_prepare}= $mysql_server_prepare_save &&
+        return $dbh->DBI::set_err($sponge->err(), $sponge->errstr()));
+
+  $dbh->{mysql_server_prepare}= $mysql_server_prepare_save;
+
+  return $sth;
 }
 
 
